@@ -1,6 +1,6 @@
 /* 
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2009 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2010 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -25,7 +25,7 @@
  * SUCH DAMAGE.
  */
 
-const char copyright[] = "Copyright (c) 2006-2009 Roy Marples";
+const char copyright[] = "Copyright (c) 2006-2010 Roy Marples";
 
 #include <sys/file.h>
 #include <sys/socket.h>
@@ -142,7 +142,7 @@ read_pid(void)
 static void
 usage(void)
 {
-	printf("usage: "PACKAGE" [-dgknpqwxyADEGHKLOTV] [-c script] [-f file]"
+	printf("usage: "PACKAGE" [-dgknpqwxyADEGHJKLOTV] [-c script] [-f file]"
 	    " [-e var=val]\n"
 	    "              [-h hostname] [-i classID ] [-l leasetime]"
 	    " [-m metric] [-o option]\n"
@@ -231,6 +231,7 @@ close_sockets(struct interface *iface)
 		iface->raw_fd = -1;
 	}
 	if (iface->udp_fd != -1) {
+		/* we don't listen to events on the udp */
 		close(iface->udp_fd);
 		iface->udp_fd = -1;
 	}
@@ -271,6 +272,22 @@ stop_interface(struct interface *iface)
 		exit(EXIT_FAILURE);
 }
 
+static uint32_t
+dhcp_xid(struct interface *iface)
+{
+	uint32_t xid;
+
+	if (iface->state->options->options & DHCPCD_XID_HWADDR &&
+	    iface->hwlen >= sizeof(xid)) 
+		/* The lower bits are probably more unique on the network */
+		memcpy(&xid, (iface->hwaddr + iface->hwlen) - sizeof(xid),
+		    sizeof(xid));
+	else
+		xid = arc4random();
+
+	return xid;
+}
+
 static void
 send_message(struct interface *iface, int type,
     void (*callback)(void *))
@@ -387,6 +404,7 @@ start_expire(void *arg)
 	iface->state->interval = 0;
 	if (iface->addr.s_addr == 0) {
 		/* We failed to reboot, so enter discovery. */
+		iface->state->lease.addr.s_addr = 0;
 		start_discover(iface);
 		return;
 	}
@@ -523,6 +541,7 @@ handle_dhcp(struct interface *iface, struct dhcp_message **dhcpp)
 	{
 		lease->frominfo = 0;
 		lease->addr.s_addr = dhcp->yiaddr;
+		lease->cookie = dhcp->cookie;
 		if (type == 0 ||
 		    get_option_addr(&lease->server, dhcp, DHO_SERVERID) != 0)
 			lease->server.s_addr = INADDR_ANY;
@@ -693,17 +712,20 @@ handle_dhcp_packet(void *arg)
 static void
 open_sockets(struct interface *iface)
 {
-	if (iface->udp_fd != -1)
-		close(iface->udp_fd);
-	if (open_udp_socket(iface) == -1 &&
-	    (errno != EADDRINUSE || iface->addr.s_addr != 0))
-		syslog(LOG_ERR, "%s: open_udp_socket: %m", iface->name);
-	if (iface->raw_fd != -1)
-		delete_event(iface->raw_fd);
-	if (open_socket(iface, ETHERTYPE_IP) == -1)
-		syslog(LOG_ERR, "%s: open_socket: %m", iface->name);
-	if (iface->raw_fd != -1)
-		add_event(iface->raw_fd, handle_dhcp_packet, iface);
+	if (iface->raw_fd == -1) {
+		if (open_socket(iface, ETHERTYPE_IP) == -1)
+			syslog(LOG_ERR, "%s: open_socket: %m", iface->name);
+		else
+			add_event(iface->raw_fd, handle_dhcp_packet, iface);
+	}
+	if (iface->udp_fd == -1 &&
+	    iface->addr.s_addr != 0 &&
+	    iface->state->new != NULL &&
+	    iface->state->new->cookie == htonl(MAGIC_COOKIE))
+	{
+		if (open_udp_socket(iface) == -1 && errno != EADDRINUSE)
+			syslog(LOG_ERR, "%s: open_udp_socket: %m", iface->name);
+	}
 }
 
 static void
@@ -711,13 +733,13 @@ send_release(struct interface *iface)
 {
 	struct timespec ts;
 
-	if (iface->state->lease.addr.s_addr &&
-	    !IN_LINKLOCAL(htonl(iface->state->lease.addr.s_addr)))
+	if (iface->state->new != NULL &&
+	    iface->state->new->cookie == htonl(MAGIC_COOKIE))
 	{
 		syslog(LOG_INFO, "%s: releasing lease of %s",
 		    iface->name, inet_ntoa(iface->state->lease.addr));
 		open_sockets(iface);
-		iface->state->xid = arc4random();
+		iface->state->xid = dhcp_xid(iface);
 		send_message(iface, DHCP_RELEASE, NULL);
 		/* Give the packet a chance to go before dropping the ip */
 		ts.tv_sec = RELEASE_DELAY_S;
@@ -760,6 +782,15 @@ configure_interface1(struct interface *iface)
 	if (iface->hwlen > DHCP_CHADDR_LEN)
 		ifo->options |= DHCPCD_CLIENTID;
 
+	/* Firewire and InfiniBand interfaces require ClientID and
+	 * the broadcast option being set. */
+	switch (iface->family) {
+	case ARPHRD_IEEE1394:	/* FALLTHROUGH */
+	case ARPHRD_INFINIBAND:
+		ifo->options |= DHCPCD_CLIENTID | DHCPCD_BROADCAST;
+		break;
+	}
+
 	free(iface->clientid);
 	if (*ifo->clientid) {
 		iface->clientid = xmalloc(ifo->clientid[0] + 1);
@@ -886,7 +917,7 @@ start_discover(void *arg)
 	struct if_options *ifo = iface->state->options;
 
 	iface->state->state = DHS_DISCOVER;
-	iface->state->xid = arc4random();
+	iface->state->xid = dhcp_xid(iface);
 	open_sockets(iface);
 	delete_timeout(NULL, iface);
 	if (ifo->fallback)
@@ -920,7 +951,7 @@ start_renew(void *arg)
 	syslog(LOG_INFO, "%s: renewing lease of %s",
 	    iface->name, inet_ntoa(iface->state->lease.addr));
 	iface->state->state = DHS_RENEW;
-	iface->state->xid = arc4random();
+	iface->state->xid = dhcp_xid(iface);
 	open_sockets(iface);
 	send_renew(iface);
 }
@@ -1020,7 +1051,7 @@ start_inform(struct interface *iface)
 	}
 
 	iface->state->state = DHS_INFORM;
-	iface->state->xid = arc4random();
+	iface->state->xid = dhcp_xid(iface);
 	open_sockets(iface);
 	send_inform(iface);
 }
@@ -1038,27 +1069,26 @@ start_reboot(struct interface *iface)
 		start_static(iface);
 		return;
 	}
-	if (ifo->reboot == 0) {
+	if (ifo->reboot == 0 || iface->state->offer == NULL) {
 		start_discover(iface);
 		return;
 	}
-	if (IN_LINKLOCAL(htonl(iface->state->lease.addr.s_addr))) {
+	if (ifo->options & DHCPCD_INFORM) {
+		syslog(LOG_INFO, "%s: informing address of %s",
+		    iface->name, inet_ntoa(iface->state->lease.addr));
+	} else if (iface->state->offer->cookie == 0) {
 		if (ifo->options & DHCPCD_IPV4LL) {
 			iface->state->claims = 0;
 			send_arp_announce(iface);
 		} else
 			start_discover(iface);
 		return;
-	}
-	if (ifo->options & DHCPCD_INFORM) {
-		syslog(LOG_INFO, "%s: informing address of %s",
-		    iface->name, inet_ntoa(iface->state->lease.addr));
 	} else {
 		syslog(LOG_INFO, "%s: rebinding lease of %s",
 		    iface->name, inet_ntoa(iface->state->lease.addr));
 	}
 	iface->state->state = DHS_REBOOT;
-	iface->state->xid = arc4random();
+	iface->state->xid = dhcp_xid(iface);
 	iface->state->lease.server.s_addr = 0;
 	delete_timeout(NULL, iface);
 	if (ifo->fallback)
@@ -1135,22 +1165,26 @@ start_interface(void *arg)
 	if (iface->state->offer) {
 		get_lease(&iface->state->lease, iface->state->offer);
 		iface->state->lease.frominfo = 1;
-		if (IN_LINKLOCAL(htonl(iface->state->offer->yiaddr))) {
+		if (iface->state->offer->cookie == 0) {
 			if (iface->state->offer->yiaddr ==
 			    iface->addr.s_addr)
 			{
 				free(iface->state->offer);
 				iface->state->offer = NULL;
 			}
-		} else if (stat(iface->leasefile, &st) == 0 &&
-		    get_option_uint32(&l, iface->state->offer,
-			DHO_LEASETIME) == 0)
+		} else if (iface->state->lease.leasetime != ~0U &&
+		    stat(iface->leasefile, &st) == 0)
 		{
 			/* Offset lease times and check expiry */
 			gettimeofday(&now, NULL);
-			if ((time_t)l < now.tv_sec - st.st_mtime) {
+			if ((time_t)iface->state->lease.leasetime <
+			    now.tv_sec - st.st_mtime)
+			{
+				syslog(LOG_DEBUG,
+				    "%s: discarding expired lease", iface->name);
 				free(iface->state->offer);
 				iface->state->offer = NULL;
+				iface->state->lease.addr.s_addr = 0;
 			} else {
 				l = now.tv_sec - st.st_mtime;
 				iface->state->lease.leasetime -= l;
@@ -1159,9 +1193,9 @@ start_interface(void *arg)
 			}
 		}
 	}
-	if (!iface->state->offer)
+	if (iface->state->offer == NULL)
 		start_discover(iface);
-	else if (IN_LINKLOCAL(htonl(iface->state->lease.addr.s_addr)) &&
+	else if (iface->state->offer->cookie == 0 &&
 	    iface->state->options->options & DHCPCD_IPV4LL)
 		start_ipv4ll(iface);
 	else
@@ -1302,7 +1336,7 @@ handle_ifa(int type, const char *ifname,
 		run_script(ifp);
 		if (ifo->options & DHCPCD_INFORM) {
 			ifp->state->state = DHS_INFORM;
-			ifp->state->xid = arc4random();
+			ifp->state->xid = dhcp_xid(ifp);
 			ifp->state->lease.server.s_addr =
 			    dst ? dst->s_addr : INADDR_ANY;
 			ifp->addr = *addr;
