@@ -1,6 +1,6 @@
 /* 
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2009 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2010 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -26,6 +26,7 @@
  */
 
 #include <sys/types.h>
+#include <sys/utsname.h>
 
 #include <arpa/inet.h>
 
@@ -44,6 +45,7 @@
 #include "common.h"
 #include "if-options.h"
 #include "net.h"
+#include "platform.h"
 
 /* These options only make sense in the config file, so don't use any
    valid short options for them */
@@ -72,6 +74,7 @@ const struct option cf_options[] = {
 	{"timeout",         required_argument, NULL, 't'},
 	{"userclass",       required_argument, NULL, 'u'},
 	{"vendor",          required_argument, NULL, 'v'},
+	{"waitip",          no_argument,       NULL, 'w'},
 	{"exit",            no_argument,       NULL, 'x'},
 	{"allowinterfaces", required_argument, NULL, 'z'},
 	{"reboot",          required_argument, NULL, 'y'},
@@ -82,7 +85,9 @@ const struct option cf_options[] = {
 	{"lastlease",       no_argument,       NULL, 'E'},
 	{"fqdn",            optional_argument, NULL, 'F'},
 	{"nogateway",       no_argument,       NULL, 'G'},
+	{"xidhwaddr",       no_argument,       NULL, 'H'}, 
 	{"clientid",        optional_argument, NULL, 'I'},
+	{"broadcast",       no_argument,       NULL, 'J'},
 	{"nolink",          no_argument,       NULL, 'K'},
 	{"noipv4ll",        no_argument,       NULL, 'L'},
 	{"destination",     required_argument, NULL, 'N'},
@@ -203,20 +208,24 @@ parse_string_hwaddr(char *sbuf, ssize_t slen, const char *str, int clid)
 		}
 		if (*str == '\\') {
 			str++;
-			switch(*str++) {
+			switch(*str) {
 			case '\0':
 				break;
 			case 'b':
 				*sbuf++ = '\b';
+				str++;
 				break;
 			case 'n':
 				*sbuf++ = '\n';
+				str++;
 				break;
 			case 'r':
 				*sbuf++ = '\r';
+				str++;
 				break;
 			case 't':
 				*sbuf++ = '\t';
+				str++;
 				break;
 			case 'x':
 				/* Grab a hex code */
@@ -254,8 +263,10 @@ parse_string_hwaddr(char *sbuf, ssize_t slen, const char *str, int clid)
 		} else
 			*sbuf++ = *str++;
 	}
-	if (punt_last)
+	if (punt_last) {
 		*--sbuf = '\0';
+		l--;
+	}
 	return l;
 }
 
@@ -450,6 +461,27 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 			syslog(LOG_ERR, "invalid vendor format");
 			return -1;
 		}
+
+		/* If vendor starts with , then it is not encapsulated */
+		if (p == arg) {
+			arg++;
+			s = parse_string((char *)ifo->vendor + 1,
+			    VENDOR_MAX_LEN, arg);
+			if (s == -1) {
+				syslog(LOG_ERR, "vendor: %m");
+				return -1;
+			}
+			ifo->vendor[0] = (uint8_t)s;
+			ifo->options |= DHCPCD_VENDORRAW;
+			break;
+		}
+
+		/* Encapsulated vendor options */
+		if (ifo->options & DHCPCD_VENDORRAW) {
+			ifo->options &= ~DHCPCD_VENDORRAW;
+			ifo->vendor[0] = 0;
+		}
+
 		*p = '\0';
 		i = atoint(arg);
 		arg = p + 1;
@@ -480,6 +512,9 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 			ifo->vendor[0] += s + 2;
 		}
 		break;
+	case 'w':
+		ifo->options |= DHCPCD_WAITIP;
+		break;
 	case 'y':
 		ifo->reboot = atoint(arg);
 		if (ifo->reboot < 0) {
@@ -537,6 +572,9 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 	case 'G':
 		ifo->options &= ~DHCPCD_GATEWAY;
 		break;
+	case 'H':
+		ifo->options |= DHCPCD_XID_HWADDR;
+		break;
 	case 'I':
 		/* Strings have a type of 0 */;
 		ifo->clientid[1] = 0;
@@ -552,6 +590,9 @@ parse_option(struct if_options *ifo, int opt, const char *arg)
 		ifo->options |= DHCPCD_CLIENTID;
 		ifo->clientid[0] = (uint8_t)s;
 		break;
+	case 'J':
+		ifo->options |= DHCPCD_BROADCAST;
+		break;
 	case 'K':
 		ifo->options &= ~DHCPCD_LINK;
 		break;
@@ -732,8 +773,9 @@ read_config(const char *file,
 {
 	struct if_options *ifo;
 	FILE *f;
-	char *line, *option, *p;
+	char *line, *option, *p, *platform;
 	int skip = 0, have_profile = 0;
+	struct utsname utn;
 
 	/* Seed our default options */
 	ifo = xzalloc(sizeof(*ifo));
@@ -749,9 +791,17 @@ read_config(const char *file,
 	if (strcmp(ifo->hostname, "(none)") == 0 ||
 	    strcmp(ifo->hostname, "localhost") == 0)
 		ifo->hostname[0] = '\0';
-	ifo->vendorclassid[0] = snprintf((char *)ifo->vendorclassid + 1,
-	    VENDORCLASSID_MAX_LEN,
-	    "%s %s", PACKAGE, VERSION);
+
+	platform = hardware_platform();
+	if (uname(&utn) == 0)
+		ifo->vendorclassid[0] = snprintf((char *)ifo->vendorclassid + 1,
+		    VENDORCLASSID_MAX_LEN,
+	            "%s-%s:%s-%s:%s%s%s", PACKAGE, VERSION,
+		    utn.sysname, utn.release, utn.machine,
+		    platform ? ":" : "", platform);
+	else
+		ifo->vendorclassid[0] = snprintf((char *)ifo->vendorclassid + 1,
+		    VENDORCLASSID_MAX_LEN, "%s-%s", PACKAGE, VERSION);
 
 	/* Parse our options file */
 	f = fopen(file ? file : CONFIG, "r");
@@ -810,7 +860,7 @@ read_config(const char *file,
 	}
 
 	/* Terminate the encapsulated options */
-	if (ifo && ifo->vendor[0]) {
+	if (ifo && ifo->vendor[0] && !(ifo->options & DHCPCD_VENDORRAW)) {
 		ifo->vendor[0]++;
 		ifo->vendor[ifo->vendor[0]] = DHO_END;
 	}
@@ -830,7 +880,7 @@ add_options(struct if_options *ifo, int argc, char **argv)
 			break;
 	}
 	/* Terminate the encapsulated options */
-	if (r == 1 && ifo->vendor[0]) {
+	if (r == 1 && ifo->vendor[0] && !(ifo->options & DHCPCD_VENDORRAW)) {
 		ifo->vendor[0]++;
 		ifo->vendor[ifo->vendor[0]] = DHO_END;
 	}
