@@ -439,10 +439,24 @@ discover_interfaces(int argc, char * const *argv)
 				free_interface(ifp);
 				continue;
 			}
-			if (ifp->family != ARPHRD_IEEE1394)
+			switch (ifp->family) {
+			case ARPHRD_IEEE1394: /* FALLTHROUGH */
+			case ARPHRD_INFINIBAND:
+				/* We don't warn for supported families */
+				break;
+			default:
 				syslog(LOG_WARNING,
 				    "%s: unknown hardware family", p);
+			}
 		}
+
+		/* Handle any platform init for the interface */
+		if (if_init(ifp) == -1) {
+			syslog(LOG_ERR, "%s: if_init: %m", p);
+			free_interface(ifp);
+			continue;
+		}
+
 		if (ifl)
 			ifl->next = ifp; 
 		else
